@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 
 import os
+import glob
 import shutil
 import argparse
 from Bio import SeqIO
@@ -11,7 +12,7 @@ from distutils.spawn import find_executable
 matam_16s_usage = '''
 =================================== matam_16s example commands ===================================
 
-MarkerMAG matam_16s -p Test -in combined_R1_R2.fasta -pct 0.1,0.5,1,5,10,25,50,75 -i 0.995 -t 12 -force -ref path/to/SILVA_128_SSURef_NR95 -matam_assembly path/to/matam_assembly.py -sortmerna /path/to/sortmerna
+MarkerMAG matam_16s -p Test -r1 R1.fasta -r2 R2.fasta -t 12 -force -d path/to/SILVA_128_SSURef_NR95
 
 ==================================================================================================
 '''
@@ -61,7 +62,7 @@ def str_to_num_list(nums_str):
         else:
             subsample_pct_list.append(float(pct_value))
 
-    return subsample_pct_list
+    return sorted(subsample_pct_list)
 
 
 def sep_paired_and_singleton_reads(fasta_in, fasta_out_r1, fasta_out_r2, fasta_out_singleton):
@@ -149,8 +150,9 @@ def subsample_paired_and_singleton_reads(paired_r1, paired_r2, singleton_in, sub
 def subsample_sortmerna_output(sortmerna_op, subsample_pct, sortmerna_op_subsampled, usearch_exe, seqtk_exe):
 
     # define file name
-    fasta_in_path, fasta_in_basename, fasta_in_ext = sep_path_basename_ext(sortmerna_op)
+    fasta_in_path,  fasta_in_basename,  fasta_in_ext  = sep_path_basename_ext(sortmerna_op)
     fasta_out_path, fasta_out_basename, fasta_out_ext = sep_path_basename_ext(sortmerna_op_subsampled)
+
     fasta_in_paired_r1               = '%s/%s_%s_paired_r1%s'            % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)
     fasta_in_paired_r2               = '%s/%s_%s_paired_r2%s'            % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)
     fasta_in_singleton               = '%s/%s_%s_singleton%s'            % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)
@@ -196,14 +198,9 @@ def prefix_seq(seq_in, prefix, seq_out):
     seq_out_handle.close()
 
 
-def parse_uclust_output(seq_file_in, uclust_output_table, seq_file_out, cluster_to_member_file):
-
-    seq_len_dict = {}
-    for seq_record in SeqIO.parse(seq_file_in, 'fasta'):
-        seq_len_dict[seq_record.id] = len(seq_record.seq)
+def parse_uclust_output(uclust_output_table, cluster_to_member_file):
 
     cluster_id_set = set()
-    cluster_to_rep_seq_dict = {}
     cluster_to_seq_member_dict = {}
     for each_line in open(uclust_output_table):
         each_line_split = each_line.strip().split('\t')
@@ -211,12 +208,9 @@ def parse_uclust_output(seq_file_in, uclust_output_table, seq_file_out, cluster_
         seq_id = each_line_split[8].split(' ')[0]
         cluster_id_set.add(int(cluster_id))
 
-        if cluster_id not in cluster_to_rep_seq_dict:
-            cluster_to_rep_seq_dict[cluster_id] = seq_id
+        if cluster_id not in cluster_to_seq_member_dict:
             cluster_to_seq_member_dict[cluster_id] = {seq_id}
         else:
-            if seq_len_dict[seq_id] > seq_len_dict[cluster_to_rep_seq_dict[cluster_id]]:
-                cluster_to_rep_seq_dict[cluster_id] = seq_id
             cluster_to_seq_member_dict[cluster_id].add(seq_id)
 
     # write out cluster sequence members
@@ -225,18 +219,6 @@ def parse_uclust_output(seq_file_in, uclust_output_table, seq_file_out, cluster_
         cluster_to_member_file_handle.write('Cluster_%s\t%s\n' % (each_cluster, ','.join(sorted([i for i in cluster_to_seq_member_dict[str(each_cluster)]]))))
     cluster_to_member_file_handle.close()
 
-    # write out the longest sequence in each cluster
-    rep_seq_id_set = set()
-    for i in cluster_to_rep_seq_dict:
-        rep_seq_id_set.add(cluster_to_rep_seq_dict[i])
-
-    rep_seq_file_handle = open(seq_file_out, 'w')
-    for each_seq in SeqIO.parse(seq_file_in, 'fasta'):
-        if each_seq.id in rep_seq_id_set:
-            rep_seq_file_handle.write('>%s\n' % each_seq.id)
-            rep_seq_file_handle.write('%s\n' % str(each_seq.seq))
-    rep_seq_file_handle.close()
-
 
 def matam_16s(args):
 
@@ -244,14 +226,15 @@ def matam_16s(args):
 
     # file in
     output_prefix                   = args['p']
-    metagenomic_reads_paired        = args['in']
+    reads_file_r1                   = args['r1']
+    reads_file_r2                   = args['r2']
+    input_16S_reads                 = args['r16s']
     subsample_pcts                  = args['pct']
-    matam_ref                       = args['ref']
+    matam_ref                       = args['d']
     uclust_iden_cutoff              = args['i']
     num_threads                     = args['t']
     force_overwrite                 = args['force']
     matam_assembly_script           = args['matam_assembly']
-    sortmerna_exe                   = args['sortmerna']
     usearch_exe                     = args['usearch']
     seqtk_exe                       = args['seqtk']
     keep_quiet                      = args['quiet']
@@ -259,7 +242,7 @@ def matam_16s(args):
 
     ################################################ check dependencies ################################################
 
-    program_list = [usearch_exe, seqtk_exe, sortmerna_exe, matam_assembly_script]
+    program_list = [usearch_exe, seqtk_exe, matam_assembly_script]
 
     not_detected_programs = []
     for needed_program in program_list:
@@ -273,46 +256,80 @@ def matam_16s(args):
 
     ####################################################################################################################
 
-    subsample_pct_list  = str_to_num_list(subsample_pcts)
-    sortmerna_ref       = '%s.clustered.fasta,%s.clustered' % (matam_ref, matam_ref)
-    matam16s_wd         = '%s_Matam16S_wd'                  % (output_prefix)
-    log_file            = '%s/%s_matam_16s.log'             % (matam16s_wd, output_prefix)
-    sortmerna_op_fasta  = '%s/%s.fasta'                     % (matam16s_wd, output_prefix)
-    sortmerna_stdout    = '%s/%s.SortMeRNA_stdout.txt'      % (matam16s_wd, output_prefix)
+    extract_16s_reads = True
+    if (reads_file_r1 is None) and (reads_file_r2 is None) and (input_16S_reads is not None):
+        if os.path.isfile(input_16S_reads) is True:
+            extract_16s_reads = False
+        else:
+            print('%s not found, program exited!' % os.path.basename(input_16S_reads))
+            exit()
+    elif (reads_file_r1 is not None) and (reads_file_r2 is not None) and (input_16S_reads is None):
+        if os.path.isfile(reads_file_r1) is False:
+            print('%s not found, program exited!' % os.path.basename(reads_file_r1))
+            exit()
+        if os.path.isfile(reads_file_r2) is False:
+            print('%s not found, program exited!' % os.path.basename(reads_file_r2))
+            exit()
+    else:
+        print('please either provide paired reads (-r1 and -r2) or 16S reads extracted from the paired reads (-r16s), program exited!')
+        exit()
 
+    ####################################################################################################################
+
+    matam16s_wd                             = '%s_Matam16S_wd'                         % output_prefix
+    log_file                                = '%s/%s_matam_16s.log'                    % (matam16s_wd, output_prefix)
+    combined_r1_r2                          = '%s/%s_combined_R1_R2.fasta'             % (matam16s_wd, output_prefix)
+    extracted_16S_reads                     = '%s/%s_16S_reads.fasta'                  % (matam16s_wd, output_prefix)
+    combined_all_depth_matam_assemblies     = '%s/%s_assembled_16S_unclustered.fasta'  % (matam16s_wd, output_prefix)
+    uclust_output_uc                        = '%s/%s_assembled_16S_uclust_%s.uc'       % (matam16s_wd, output_prefix, uclust_iden_cutoff)
+    uclust_output_txt                       = '%s/%s_assembled_16S_uclust_%s.txt'      % (matam16s_wd, output_prefix, uclust_iden_cutoff)
+    uclust_output_fasta                     = '%s/%s_assembled_16S_uclust_%s.fasta'    % (matam16s_wd, output_prefix, uclust_iden_cutoff)
 
     # create folder
     if (os.path.isdir(matam16s_wd) is True) and (force_overwrite is False):
         print('Output folder detected, program exited: %s' % matam16s_wd)
         exit()
     else:
-        force_create_folder(matam16s_wd)
+        os.mkdir(matam16s_wd)
+
+    if extract_16s_reads is True:
+        # combine R1 and R2
+        report_and_log(('Combining the forward and reverse reads'), log_file, keep_quiet)
+        combined_r1_r2_cmd = '%s mergepe %s %s > %s' % (seqtk_exe, reads_file_r1, reads_file_r2, combined_r1_r2)
+        os.system(combined_r1_r2_cmd)
+
+        # Extract 16S reads
+        report_and_log(('Extracting 16S reads with Matam'), log_file, keep_quiet)
+        matam_filter_cmd = '%s -i %s -o %s/%s_get_16S_reads_wd --cpu %s --max_memory 100000 -v --filter_only -d %s' % (matam_assembly_script, combined_r1_r2, matam16s_wd, output_prefix, num_threads, matam_ref)
+        report_and_log(matam_filter_cmd, log_file, True)
+        os.system(matam_filter_cmd)
+
+        # get file name of extracted 16S reads
+        matam_16S_reads_re = '%s/%s_get_16S_reads_wd/workdir/*.fasta' % (matam16s_wd, output_prefix)
+        matam_16S_reads    = glob.glob(matam_16S_reads_re)[0]
+        os.system('mv %s %s' % (matam_16S_reads, extracted_16S_reads))
+
+    else:
+        extracted_16S_reads = input_16S_reads
+        #os.system('cp %s %s' % (input_16S_reads, extracted_16S_reads))
 
 
-    # run SortMeRNA
-    report_and_log(('Extracting rRNA reads with SortMeRNA from %s' % metagenomic_reads_paired), log_file, keep_quiet)
-
-    sortmerna_cmd = '%s --ref %s --reads %s --aligned %s/%s --fastx --sam --blast "1" --log --best 10 --min_lis 10 -e 1.00e-05 -a %s -v > %s' % (sortmerna_exe, sortmerna_ref, metagenomic_reads_paired, matam16s_wd, output_prefix, num_threads, sortmerna_stdout)
-    report_and_log(sortmerna_cmd, log_file, True)
-    os.system(sortmerna_cmd)
-
-
-    # subsample SortMeRNA output and assemble
+    # subsample 16S reads and assemble
     renamed_matam_assembly_list = []
-    for subsample_pct in subsample_pct_list:
+    for subsample_pct in str_to_num_list(subsample_pcts):
 
         report_and_log(('Subsample RNA reads at %s percent' % subsample_pct), log_file, keep_quiet)
 
-        subsample_reads_file = '%s/%s_subsample_%s.fasta'    % (matam16s_wd, output_prefix, subsample_pct)
-        matam_output_folder  = '%s/%s_subsample_%s_Matam_wd' % (matam16s_wd, output_prefix, subsample_pct)
+        subsample_reads_file = '%s/%s_16S_reads_subset_%s.fasta'    % (matam16s_wd, output_prefix, subsample_pct)
+        matam_output_folder  = '%s/%s_16S_reads_subset_%s_Matam_wd' % (matam16s_wd, output_prefix, subsample_pct)
 
         # subsample
-        subsample_sortmerna_output(sortmerna_op_fasta, subsample_pct, subsample_reads_file, usearch_exe, seqtk_exe)
+        subsample_sortmerna_output(extracted_16S_reads, subsample_pct, subsample_reads_file, usearch_exe, seqtk_exe)
 
         # assemble with Matam
         report_and_log(('Assembling subsampled reads with Matam'), log_file, keep_quiet)
 
-        matam_cmd = 'python3 %s -d %s -i %s --cpu %s --max_memory 100000 -v -o %s' % (matam_assembly_script, matam_ref, subsample_reads_file, num_threads, matam_output_folder)
+        matam_cmd = '%s -d %s -i %s --cpu %s --max_memory 100000 -v -o %s' % (matam_assembly_script, matam_ref, subsample_reads_file, num_threads, matam_output_folder)
         report_and_log(matam_cmd, log_file, True)
         os.system(matam_cmd)
 
@@ -321,84 +338,54 @@ def matam_16s(args):
         seq_prefix                = '%s_subsample_%s'                               % (output_prefix, subsample_pct)
 
         if os.path.isfile(matam_assemblies) is True:
-
             report_and_log(('Adding prefix to Matam assemblies'), log_file, keep_quiet)
-
             prefix_seq(matam_assemblies, seq_prefix, matam_assemblies_prefixed)
-
             renamed_matam_assembly_list.append(matam_assemblies_prefixed)
         else:
             report_and_log(('No 16S rRNA gene sequence reconstructed at current depth!'), log_file, keep_quiet)
 
-
-    # assemble with Matam without subsample
-    report_and_log(('Assembling nonsubsampled RNA reads'), log_file, keep_quiet)
-    matam_output_folder_no_subsample   = '%s/%s_nonsubsampled_Matam_wd'                                 % (matam16s_wd, output_prefix)
-    matam_cmd_no_subsample             = 'python3 %s -d %s -i %s --cpu %s --max_memory 100000 -v -o %s' % (matam_assembly_script, matam_ref, sortmerna_op_fasta, num_threads, matam_output_folder_no_subsample)
-    report_and_log(matam_cmd_no_subsample, log_file, True)
-    os.system(matam_cmd_no_subsample)
-
-
-    # prefix no subsampled Matam assemblies
-    matam_assemblies_no_subsample          = '%s/workdir/scaffolds.NR.min_500bp.fa'          % (matam_output_folder_no_subsample)
-    matam_assemblies_no_subsample_prefixed = '%s/workdir/scaffolds.NR.min_500bp.prefixed.fa' % (matam_output_folder_no_subsample)
-    seq_prefix                             = '%s_no_subsample'                               % (output_prefix)
-    prefix_seq(matam_assemblies_no_subsample, seq_prefix, matam_assemblies_no_subsample_prefixed)
-    renamed_matam_assembly_list.append(matam_assemblies_no_subsample_prefixed)
-
-
     # combine Matam outputs
     report_and_log(('Combine Matam assemblies at all depth'), log_file, keep_quiet)
-    combined_all_depth_matam_assemblies    = '%s/%s_all_depth_assemblies.fasta' % (matam16s_wd, output_prefix)
     combine_cmd = 'cat %s > %s' % (' '.join(renamed_matam_assembly_list), combined_all_depth_matam_assemblies)
     report_and_log(combine_cmd, log_file, True)
     os.system(combine_cmd)
 
-
     # dereplicate combined assemblies with Usearch
-    report_and_log(('Dereplicate combined Matam assemblies at %s identity cutoff' % uclust_iden_cutoff), log_file, keep_quiet)
-    uclust_output_fasta             = '%s/%s_all_depth_assemblies.dereplicated.fasta'                       % (matam16s_wd, output_prefix)
-    uclust_output_table             = '%s/%s_all_depth_assemblies.uc'                                       % (matam16s_wd, output_prefix)
-    cluster_to_member_file          = '%s/%s_all_depth_assemblies.uc.reorganised.txt'                       % (matam16s_wd, output_prefix)
-    default_centroids_to_be_ignored = '%s/%s_all_depth_assemblies_default_centroids.fasta'                  % (matam16s_wd, output_prefix)
-    uclust_cmd                      = '%s -cluster_fast %s -id %s -centroids %s -uc %s -sort length -quiet' % (usearch_exe, combined_all_depth_matam_assemblies, uclust_iden_cutoff, default_centroids_to_be_ignored, uclust_output_table)
+    report_and_log(('Cluster combined Matam assemblies with UCLUST at %s identity cutoff' % uclust_iden_cutoff), log_file, keep_quiet)
+    uclust_cmd = '%s -cluster_fast %s -id %s -centroids %s -uc %s -sort length -quiet' % (usearch_exe, combined_all_depth_matam_assemblies, uclust_iden_cutoff, uclust_output_fasta, uclust_output_uc)
     report_and_log(uclust_cmd, log_file, True)
     os.system(uclust_cmd)
 
+    # get readable cluster results
+    parse_uclust_output(uclust_output_uc, uclust_output_txt)
 
-    # parse_uclust_output
-    parse_uclust_output(combined_all_depth_matam_assemblies, uclust_output_table, uclust_output_fasta, cluster_to_member_file)
-
+    # remove tmp file
+    os.system('rm %s' % combined_r1_r2)
 
     # report
-    report_and_log(('Dereplicated Matam assemblies exported to %s' % uclust_output_fasta), log_file, keep_quiet)
+    report_and_log(('SortMeRNA identified 16S reads exported to %s' % os.path.basename(extracted_16S_reads)), log_file, keep_quiet)
+    report_and_log(('Dereplicated Matam assemblies exported to %s'  % os.path.basename(uclust_output_fasta)), log_file, keep_quiet)
+    report_and_log(('Done!'), log_file, keep_quiet)
 
 
 if __name__ == '__main__':
 
     matam_16s_parser = argparse.ArgumentParser()
 
-    matam_16s_parser.add_argument('-p',                 required=True,                                       help='output prefix')
-    matam_16s_parser.add_argument('-in',                required=True,                                       help='combined reads file')
-    matam_16s_parser.add_argument('-pct',               required=True,  type=str, default='1,5,10,25,50,75', help='subsample percentage, deafault: 1,5,10,25,50,75')
-    matam_16s_parser.add_argument('-ref',               required=False, type=str,                            help='Path to Matam reference database')
-    matam_16s_parser.add_argument('-i',                 required=False, type=float, default=0.995,           help='cluster identity cutoff (0-1), default: 0.995')
-    matam_16s_parser.add_argument('-t',                 required=False, type=int, default=1,                 help='number of threads, default: 1')
-    matam_16s_parser.add_argument('-force',             required=False, action="store_true",                 help='force overwrite existing results')
-    matam_16s_parser.add_argument('-quiet',             required=False, action="store_true",                 help='not report progress')
-    matam_16s_parser.add_argument('-matam_assembly',    required=False, type=str,                            help='path to matam_assembly.py')
-    matam_16s_parser.add_argument('-sortmerna',         required=False, type=str, default='sortmerna',       help='path to sortmerna executable file, default: sortmerna')
-    matam_16s_parser.add_argument('-seqtk',             required=False, type=str, default='seqtk',           help='path to seqtk executable file, default: seqtk')
-    matam_16s_parser.add_argument('-usearch',           required=False, type=str, default='usearch',         help='path to usearch executable file, default: usearch')
+    matam_16s_parser.add_argument('-p',              required=True,                                                     help='output prefix')
+    matam_16s_parser.add_argument('-r1',             required=False, default=None,                                      help='forward reads')
+    matam_16s_parser.add_argument('-r2',             required=False, default=None,                                      help='reverse reads')
+    matam_16s_parser.add_argument('-r16s',           required=False, default=None,                                      help='extracted 16S reads')
+    matam_16s_parser.add_argument('-pct',            required=True,  type=str, default='1,5,10,25,50,75,100',           help='subsample percentage, must be integer, between 1-100, deafault: 1,5,10,25,50,75,100')
+    matam_16s_parser.add_argument('-d',              required=False, type=str,                                          help='MATAM ref db, same as "-d" in Matam')
+    matam_16s_parser.add_argument('-i',              required=False, type=float, default=0.995,                         help='cluster identity cutoff (0-1), default: 0.995')
+    matam_16s_parser.add_argument('-t',              required=False, type=int, default=1,                               help='number of threads, default: 1')
+    matam_16s_parser.add_argument('-force',          required=False, action="store_true",                               help='force overwrite existing results')
+    matam_16s_parser.add_argument('-quiet',          required=False, action="store_true",                               help='not report progress')
+    matam_16s_parser.add_argument('-matam_assembly', required=False, type=str, default='matam_assembly.py',             help='path to matam_assembly.py, default: matam_assembly.py')
+    matam_16s_parser.add_argument('-seqtk',          required=False, type=str, default='seqtk',                         help='path to seqtk executable file, default: seqtk')
+    matam_16s_parser.add_argument('-usearch',        required=False, type=str, default='usearch',                       help='path to usearch executable file, default: usearch')
 
     args = vars(matam_16s_parser.parse_args())
-
     matam_16s(args)
 
-'''
-
-To do:
-1. support to customize parameters for Matam
-2. silent Matam
-
-'''
